@@ -1,7 +1,16 @@
 # -*- coding: utf-8 -*-
 #
 # Picard, the next-generation MusicBrainz tagger
-# Copyright (C) 2006 Lukáš Lalinský
+#
+# Copyright (C) 2006-2007, 2014, 2017 Lukáš Lalinský
+# Copyright (C) 2008, 2014, 2019-2020 Philipp Wolfer
+# Copyright (C) 2012, 2017 Wieland Hoffmann
+# Copyright (C) 2012-2014 Michael Wiencek
+# Copyright (C) 2013-2016, 2018-2019 Laurent Monin
+# Copyright (C) 2016 Suhas
+# Copyright (C) 2016-2018 Sambhav Kothari
+# Copyright (C) 2017 Sophist-UK
+# Copyright (C) 2018 Vishal Choudhary
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,7 +26,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
+
 from operator import itemgetter
+import os
+import shutil
 
 from PyQt5 import QtCore
 
@@ -26,10 +38,9 @@ from picard import (
     PICARD_ORG_NAME,
     PICARD_VERSION,
     log,
-    version_from_string,
-    version_to_string,
 )
 from picard.util import LockableObject
+from picard.version import Version
 
 
 class ConfigUpgradeError(Exception):
@@ -43,25 +54,17 @@ class ConfigSection(LockableObject):
     def __init__(self, config, name):
         super().__init__()
         self.__qt_config = config
-        self.__config = {}
         self.__name = name
-        self.__load_keys()
+        self.__prefix = self.__name + '/'
+        self.__prefix_len = len(self.__prefix)
 
-    def __qt_keys(self):
-        prefix = self.__name + '/'
-        return filter(lambda key: key.startswith(prefix),
-                      self.__qt_config.allKeys())
+    def key(self, name):
+        return self.__prefix + name
 
-    def __load_keys(self):
-        for key in self.__qt_keys():
-            try:
-                self.__config[key] = self.__qt_config.value(key)
-            except TypeError:
-                # Related to PICARD-1255, Unable to load the object into
-                # Python at all. Something weird with the way it is read and converted
-                # via the Qt C++ API. Simply ignore the key and it will be reset to
-                # default whenever the user opens Picard options
-                log.error('Unable to load config value: %s', key)
+    def _subkeys(self):
+        for key in self.__qt_config.allKeys():
+            if key[:self.__prefix_len] == self.__prefix:
+                yield key[self.__prefix_len:]
 
     def __getitem__(self, name):
         opt = Option.get(self.__name, name)
@@ -70,42 +73,41 @@ class ConfigSection(LockableObject):
         return self.value(name, opt, opt.default)
 
     def __setitem__(self, name, value):
-        key = self.__name + '/' + name
         self.lock_for_write()
         try:
-            self.__config[key] = value
-            self.__qt_config.setValue(key, value)
+            self.__qt_config.setValue(self.key(name), value)
         finally:
             self.unlock()
 
     def __contains__(self, name):
-        key = self.__name + '/' + name
-        return key in self.__config
+        return self.__qt_config.contains(self.key(name))
 
     def remove(self, name):
-        key = self.__name + '/' + name
         self.lock_for_write()
         try:
-            if key in self.__config:
-                self.__config.pop(key)
-                self.__qt_config.remove(key)
+            if name in self:
+                self.__qt_config.remove(self.key(name))
         finally:
             self.unlock()
 
-    def raw_value(self, name):
+    def raw_value(self, name, qtype=None):
         """Return an option value without any type conversion."""
-        value = self.__config[self.__name + '/' + name]
-        return value
+        key = self.key(name)
+        if qtype is not None:
+            return self.__qt_config.value(key, type=qtype)
+        else:
+            return self.__qt_config.value(key)
 
     def value(self, name, option_type, default=None):
         """Return an option value converted to the given Option type."""
-        key = self.__name + '/' + name
         self.lock_for_read()
         try:
-            if key in self.__config:
-                return option_type.convert(self.raw_value(name))
+            if name in self:
+                value = self.raw_value(name, qtype=option_type.qtype)
+                return option_type.convert(value)
             return default
-        except Exception:
+        except Exception as why:
+            log.error('Cannot read %s value: %s', self.key(name), why, exc_info=True)
             return default
         finally:
             self.unlock()
@@ -116,20 +118,12 @@ class Config(QtCore.QSettings):
     """Configuration."""
 
     def __init__(self):
-        pass
+        self.__known_keys = set()
 
     def __initialize(self):
         """Common initializer method for :meth:`from_app` and
         :meth:`from_file`."""
 
-        # If there are no settings, copy existing settings from old format
-        # (registry on windows systems)
-        if not self.allKeys():
-            oldFormat = QtCore.QSettings(PICARD_ORG_NAME, PICARD_APP_NAME)
-            for k in oldFormat.allKeys():
-                self.setValue(k, oldFormat.value(k))
-            self.sync()
-
         self.application = ConfigSection(self, "application")
         self.setting = ConfigSection(self, "setting")
         self.persist = ConfigSection(self, "persist")
@@ -137,8 +131,14 @@ class Config(QtCore.QSettings):
         self.current_preset = "default"
 
         TextOption("application", "version", '0.0.0dev0')
-        self._version = version_from_string(self.application["version"])
+        self._version = Version.from_string(self.application["version"])
         self._upgrade_hooks = dict()
+        # Save known config names for faster access and to prevent
+        # strange cases of Qt locking up when accessing QSettings.allKeys()
+        # or QSettings.contains() shortly after having written settings
+        # inside threads.
+        # See https://tickets.metabrainz.org/browse/PICARD-1590
+        self.__known_keys = set(self.allKeys())
 
     @classmethod
     def from_app(cls, parent):
@@ -148,7 +148,22 @@ class Config(QtCore.QSettings):
         QtCore.QSettings.__init__(this, QtCore.QSettings.IniFormat,
                                   QtCore.QSettings.UserScope, PICARD_ORG_NAME,
                                   PICARD_APP_NAME, parent)
+
+        # Check if there is a config file specifically for this version
+        versioned_config_file = this._versioned_config_filename(PICARD_VERSION)
+        if os.path.isfile(versioned_config_file):
+            return cls.from_file(parent, versioned_config_file)
+
+        # If there are no settings, copy existing settings from old format
+        # (registry on windows systems)
+        if not this.allKeys():
+            oldFormat = QtCore.QSettings(PICARD_ORG_NAME, PICARD_APP_NAME)
+            for k in oldFormat.allKeys():
+                this.setValue(k, oldFormat.value(k))
+            this.sync()
+
         this.__initialize()
+        this._backup_settings()
         return this
 
     @classmethod
@@ -161,6 +176,19 @@ class Config(QtCore.QSettings):
         this.__initialize()
         return this
 
+    def setValue(self, key, value):
+        super().setValue(key, value)
+        self.__known_keys.add(key)
+
+    def remove(self, key):
+        super().remove(key)
+        self.__known_keys.discard(key)
+
+    def contains(self, key):
+        # Overwritten due to https://tickets.metabrainz.org/browse/PICARD-1590
+        # See comment above in __initialize
+        return key in self.__known_keys or super().contains(key)
+
     def switchProfile(self, profilename):
         """Sets the current profile."""
         key = "profile/%s" % (profilename,)
@@ -171,7 +199,7 @@ class Config(QtCore.QSettings):
 
     def register_upgrade_hook(self, func, *args):
         """Register a function to upgrade from one config version to another"""
-        to_version = version_from_string(func.__name__)
+        to_version = Version.from_string(func.__name__)
         assert to_version <= PICARD_VERSION, "%r > %r !!!" % (to_version, PICARD_VERSION)
         self._upgrade_hooks[to_version] = {
             'func': func,
@@ -187,8 +215,8 @@ class Config(QtCore.QSettings):
             if self._version > PICARD_VERSION:
                 print("Warning: config file %s was created by a more recent "
                       "version of Picard (current is %s)" % (
-                          version_to_string(self._version),
-                          version_to_string(PICARD_VERSION)
+                          self._version.to_string(),
+                          PICARD_VERSION.to_string()
                       ))
             return
         for version in sorted(self._upgrade_hooks):
@@ -197,17 +225,17 @@ class Config(QtCore.QSettings):
                 try:
                     if outputfunc and hook['func'].__doc__:
                         outputfunc("Config upgrade %s -> %s: %s" % (
-                                   version_to_string(self._version),
-                                   version_to_string(version),
+                                   self._version.to_string(),
+                                   version.to_string(),
                                    hook['func'].__doc__.strip()))
-                    hook['func'](*hook['args'])
+                    hook['func'](self, *hook['args'])
                 except BaseException:
                     import traceback
                     raise ConfigUpgradeError(
                         "Error during config upgrade from version %s to %s "
                         "using %s():\n%s" % (
-                            version_to_string(self._version),
-                            version_to_string(version),
+                            self._version.to_string(),
+                            version.to_string(),
                             hook['func'].__name__,
                             traceback.format_exc()
                         ))
@@ -224,16 +252,32 @@ class Config(QtCore.QSettings):
             self._version = PICARD_VERSION
             self._write_version()
 
+    def _backup_settings(self):
+        if Version(0, 0, 0) < self._version < PICARD_VERSION:
+            backup_path = self._versioned_config_filename()
+            log.info('Backing up config file to %s', backup_path)
+            try:
+                shutil.copyfile(self.fileName(), backup_path)
+            except OSError:
+                log.error('Failed backing up config file to %s', backup_path)
+
     def _write_version(self):
-        self.application["version"] = version_to_string(self._version)
+        self.application["version"] = self._version.to_string()
         self.sync()
 
+    def _versioned_config_filename(self, version=None):
+        if not version:
+            version = self._version
+        return os.path.join(os.path.dirname(self.fileName()), '%s-%s.ini' % (
+            self.applicationName(), version.to_string(short=True)))
+
 
 class Option(QtCore.QObject):
 
     """Generic option."""
 
     registry = {}
+    qtype = None
 
     def __init__(self, section, name, default):
         super().__init__()
@@ -252,18 +296,13 @@ class Option(QtCore.QObject):
 class TextOption(Option):
 
     convert = str
+    qtype = 'QString'
 
 
 class BoolOption(Option):
 
-    @staticmethod
-    def convert(value):
-        # The QSettings IniFormat saves boolean values as the strings "true"
-        # and "false". Thus, explicit boolean and string comparisons are used
-        # to determine the value. NOTE: In PyQt >= 4.8.3, QSettings.value has
-        # an optional "type" parameter that avoids this. But we still support
-        # PyQt >= 4.5, so that is not used.
-        return value is True or value == "true"
+    convert = bool
+    qtype = bool
 
 
 class IntOption(Option):
@@ -279,13 +318,7 @@ class FloatOption(Option):
 class ListOption(Option):
 
     convert = list
-
-
-class IntListOption(Option):
-
-    @staticmethod
-    def convert(values):
-        return list(map(int, values))
+    qtype = 'QVariantList'
 
 
 config = None
