@@ -1,7 +1,11 @@
 # -*- coding: utf-8 -*-
 #
 # Picard, the next-generation MusicBrainz tagger
-# Copyright (C) 2015 Laurent Monin
+#
+# Copyright (C) 2015, 2018-2019 Laurent Monin
+# Copyright (C) 2016-2017 Sambhav Kothari
+# Copyright (C) 2017 Ville Skyttä
+# Copyright (C) 2019 Philipp Wolfer
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,11 +21,12 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
+
 import os
 import re
 
 from picard import config
-from picard.coverart.image import CoverArtImageFromFile
+from picard.coverart.image import LocalFileCoverArtImage
 from picard.coverart.providers import (
     CoverArtProvider,
     ProviderOptions,
@@ -64,17 +69,13 @@ class CoverArtProviderLocal(CoverArtProvider):
 
     """Get cover art from local files"""
 
-    NAME = "Local"
+    NAME = "Local Files"
     TITLE = N_("Local Files")
     OPTIONS = ProviderOptionsLocal
 
     _types_split_re = re.compile('[^a-z0-9]', re.IGNORECASE)
     _known_types = set([t['name'] for t in CAA_TYPES])
 
-    def enabled(self):
-        enabled = CoverArtProvider.enabled(self)
-        return enabled and not self.coverart.front_image_found
-
     def queue_images(self):
         _match_re = re.compile(config.setting['local_cover_regex'], re.IGNORECASE)
         dirs_done = set()
@@ -92,9 +93,11 @@ class CoverArtProviderLocal(CoverArtProvider):
                     filepath = os.path.join(current_dir, root, filename)
                     if os.path.exists(filepath):
                         types = self.get_types(m.group(1)) or ['front']
-                        self.queue_put(CoverArtImageFromFile(filepath,
-                                                             types=types,
-                                                             support_types=True))
+                        self.queue_put(
+                            LocalFileCoverArtImage(filepath,
+                                                   types=types,
+                                                   support_types=True,
+                                                   support_multi_types=True))
         return CoverArtProvider.FINISHED
 
     def get_types(self, string):
