@@ -1,7 +1,14 @@
 # -*- coding: utf-8 -*-
 #
 # Picard, the next-generation MusicBrainz tagger
-# Copyright (C) 2007 Lukáš Lalinský
+#
+# Copyright (C) 2007, 2011 Lukáš Lalinský
+# Copyright (C) 2008-2010, 2019 Philipp Wolfer
+# Copyright (C) 2012-2013 Michael Wiencek
+# Copyright (C) 2013, 2015, 2018-2019 Laurent Monin
+# Copyright (C) 2016-2018 Sambhav Kothari
+# Copyright (C) 2017 Sophist-UK
+# Copyright (C) 2018 Wieland Hoffmann
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,6 +24,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
+
 from collections import (
     OrderedDict,
     deque,
@@ -28,6 +36,7 @@ from threading import Lock
 
 from PyQt5 import QtCore
 
+
 _MAX_TAIL_LEN = 10**6
 
 VERBOSITY_DEFAULT = logging.WARNING
@@ -41,13 +50,13 @@ def get_effective_level():
     return main_logger.getEffectiveLevel()
 
 
-_feat = namedtuple('_feat', ['name', 'prefix', 'fgcolor'])
+_feat = namedtuple('_feat', ['name', 'prefix', 'color_key'])
 
 levels_features = OrderedDict([
-    (logging.ERROR,   _feat('Error',   'E', 'red')),
-    (logging.WARNING, _feat('Warning', 'W', 'darkorange')),
-    (logging.INFO,    _feat('Info',    'I', 'black')),
-    (logging.DEBUG,   _feat('Debug',   'D', 'purple')),
+    (logging.ERROR,   _feat('Error',   'E', 'log_error')),
+    (logging.WARNING, _feat('Warning', 'W', 'log_warning')),
+    (logging.INFO,    _feat('Info',    'I', 'log_info')),
+    (logging.DEBUG,   _feat('Debug',   'D', 'log_debug')),
 ])
 
 
