@@ -1,7 +1,12 @@
 # -*- coding: utf-8 -*-
 #
 # Picard, the next-generation MusicBrainz tagger
-# Copyright (C) 2017 Sambhav Kothari
+#
+# Copyright (C) 2017 David Mandelberg
+# Copyright (C) 2017-2018 Sambhav Kothari
+# Copyright (C) 2017-2019 Laurent Monin
+# Copyright (C) 2018-2020 Philipp Wolfer
+# Copyright (C) 2019 Michael Wiencek
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,6 +22,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
+
 import re
 
 from picard import config
@@ -28,6 +34,7 @@ from picard.util import (
     translate_from_sortname,
 )
 
+
 _artist_rel_types = {
     "arranger": "arranger",
     "audio": "engineer",
@@ -264,9 +271,9 @@ def artist_credit_to_metadata(node, m, release=False):
         m["~artists_sort"] = artistssort
 
 
-def country_list_from_node(node):
+def countries_from_node(node):
+    countries = []
     if "release-events" in node:
-        country = []
         for release_event in node['release-events']:
             try:
                 country_code = release_event['area']['iso-3166-1-codes'][0]
@@ -275,8 +282,8 @@ def country_list_from_node(node):
                 pass
             else:
                 if country_code:
-                    country.append(country_code)
-        return country
+                    countries.append(country_code)
+    return countries
 
 
 def release_dates_and_countries_from_node(node):
@@ -346,7 +353,6 @@ def track_to_metadata(node, track):
             artist_credit_to_metadata(value, m)
     if m.length:
         m['~length'] = format_time(m.length)
-    track.orig_metadata.copy(m)
 
 
 def recording_to_metadata(node, m, track=None):
@@ -365,8 +371,10 @@ def recording_to_metadata(node, m, track=None):
             artist_credit_to_metadata(value, m)
             # set tags from artists
             if track:
-                for artist in value:
-                    track.append_track_artist(artist['artist']['id'])
+                for credit in value:
+                    artist = credit['artist']
+                    artist_obj = track.append_track_artist(artist['id'])
+                    add_genres_from_node(artist, artist_obj)
         elif key == 'relations':
             _relations_to_metadata(value, m)
         elif key in ('genres', 'tags') and track:
@@ -427,9 +435,9 @@ def artist_to_metadata(node, m):
                 ended = value['ended']
                 if ended and "end" in value:
                     m["enddate"] = value['end']
-        elif key == "begin_area":
+        elif key == "begin-area":
             m["beginarea"] = value['name']
-        elif key == "end_area":
+        elif key == "end-area":
             m["endarea"] = value['name']
 
 
@@ -447,8 +455,10 @@ def release_to_metadata(node, m, album=None):
             artist_credit_to_metadata(value, m, release=True)
             # set tags from artists
             if album is not None:
-                for artist in value:
-                    album.append_album_artist(artist['artist']['id'])
+                for credit in value:
+                    artist = credit['artist']
+                    artist_obj = album.append_album_artist(artist['id'])
+                    add_genres_from_node(artist, artist_obj)
         elif key == 'relations':
             _relations_to_metadata(value, m)
         elif key == 'label-info':
@@ -458,10 +468,15 @@ def release_to_metadata(node, m, album=None):
                 m['~releaselanguage'] = value['language']
             if 'script' in value:
                 m['script'] = value['script']
-        elif key in ('genres', 'tags'):
-            add_genres(value, album)
-        elif key in ('user-genres', 'user-tags'):
-            add_user_genres(value, album)
+    m['~releasecountries'] = release_countries = countries_from_node(node)
+    # The MB web service returns the first release country in the country tag.
+    # If the user has configured preferred release countries, use the first one
+    # if it is one in the complete list of release countries.
+    for country in config.setting["preferred_release_countries"]:
+        if country in release_countries:
+            m['releasecountry'] = country
+            break
+    add_genres_from_node(node, album)
 
 
 def release_group_to_metadata(node, m, release_group=None):
@@ -472,14 +487,11 @@ def release_group_to_metadata(node, m, release_group=None):
             continue
         if key in _RELEASE_GROUP_TO_METADATA:
             m[_RELEASE_GROUP_TO_METADATA[key]] = value
-        elif key in ('genres', 'tags'):
-            add_genres(value, release_group)
-        elif key in ('user-genres', 'user-tags'):
-            add_user_genres(value, release_group)
         elif key == 'primary-type':
             m['~primaryreleasetype'] = value.lower()
         elif key == 'secondary-types':
             add_secondary_release_types(value, m)
+    add_genres_from_node(node, release_group)
     if m['originaldate']:
         m['originalyear'] = m['originaldate'][:4]
     m['releasetype'] = m.getall('~primaryreleasetype') + m.getall('~secondaryreleasetype')
@@ -490,23 +502,46 @@ def add_secondary_release_types(node, m):
         m.add_unique('~secondaryreleasetype', secondary_type.lower())
 
 
+def add_genres_from_node(node, obj):
+    if obj is None:
+        return
+    if 'genres' in node:
+        add_genres(node['genres'], obj)
+    if 'tags' in node:
+        add_genres(node['tags'], obj)
+    if 'user-genres' in node:
+        add_user_genres(node['user-genres'], obj)
+    if 'user-tags' in node:
+        add_user_genres(node['user-tags'], obj)
+
+
 def add_genres(node, obj):
-    if obj is not None:
-        for tag in node:
-            key = tag['name']
-            count = tag['count']
-            if key:
-                obj.add_genre(key, count)
+    for tag in node:
+        key = tag['name']
+        count = tag['count']
+        if key:
+            obj.add_genre(key, count)
 
 
 def add_user_genres(node, obj):
-    if obj is not None:
-        for tag in node:
-            key = tag['name']
-            if key:
-                obj.add_genre(key, 1)
+    for tag in node:
+        key = tag['name']
+        if key:
+            obj.add_genre(key, 1)
 
 
 def add_isrcs_to_metadata(node, metadata):
     for isrc in node:
         metadata.add('isrc', isrc)
+
+
+def get_score(node):
+    """Returns the score attribute for a node.
+    The score is expected to be an integer between 0 and 100, it is returned as
+    a value between 0.0 and 1.0. If there is no score attribute or it has an
+    invalid value 1.0 will be returned.
+    """
+    try:
+        return int(node.get('score', 100)) / 100
+    except (TypeError, ValueError):
+        return 1.0
