@@ -1,7 +1,32 @@
 # -*- coding: utf-8 -*-
 #
 # Picard, the next-generation MusicBrainz tagger
-# Copyright (C) 2006 Lukáš Lalinský
+#
+# Copyright (C) 2006-2008, 2011-2012, 2014 Lukáš Lalinský
+# Copyright (C) 2007 Nikolai Prokoschenko
+# Copyright (C) 2008 Gary van der Merwe
+# Copyright (C) 2008 Robert Kaye
+# Copyright (C) 2008 Will
+# Copyright (C) 2008-2010, 2015, 2018-2020 Philipp Wolfer
+# Copyright (C) 2009 Carlin Mangar
+# Copyright (C) 2009 David Hilton
+# Copyright (C) 2011-2012 Chad Wilson
+# Copyright (C) 2011-2013, 2015-2017 Wieland Hoffmann
+# Copyright (C) 2011-2014 Michael Wiencek
+# Copyright (C) 2013-2014, 2017 Sophist-UK
+# Copyright (C) 2013-2020 Laurent Monin
+# Copyright (C) 2015 Ohm Patel
+# Copyright (C) 2015 samithaj
+# Copyright (C) 2016 Rahul Raturi
+# Copyright (C) 2016 Simon Legner
+# Copyright (C) 2016-2017 Sambhav Kothari
+# Copyright (C) 2017 Antonio Larrosa
+# Copyright (C) 2017 Frederik “Freso” S. Olesen
+# Copyright (C) 2018 Bob Swift
+# Copyright (C) 2018 Kartik Ohri
+# Copyright (C) 2018 Vishal Choudhary
+# Copyright (C) 2018 virusMac
+# Copyright (C) 2019 Timur Enikeev
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,11 +42,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
+
 from collections import OrderedDict
 import datetime
 from functools import partial
 import os.path
-import sys
 
 from PyQt5 import (
     QtCore,
@@ -30,12 +55,14 @@ from PyQt5 import (
 )
 
 from picard import (
+    PICARD_APP_ID,
     config,
     log,
 )
 from picard.album import Album
 from picard.cluster import Cluster
 from picard.const import PROGRAM_UPDATE_LEVELS
+from picard.const.sys import IS_MACOS
 from picard.file import File
 from picard.formats import supported_formats
 from picard.plugin import ExtensionPoint
@@ -73,16 +100,18 @@ from picard.ui.passworddialog import (
     PasswordDialog,
     ProxyDialog,
 )
+from picard.ui.playertoolbar import Player
 from picard.ui.searchdialog.album import AlbumSearchDialog
 from picard.ui.searchdialog.track import TrackSearchDialog
+from picard.ui.statusindicator import DesktopStatusIndicator
 from picard.ui.tagsfromfilenames import TagsFromFileNamesDialog
 from picard.ui.util import (
-    ButtonLineEdit,
     MultiDirsSelectDialog,
     find_starting_directory,
 )
 
-ui_init = ExtensionPoint()
+
+ui_init = ExtensionPoint(label='ui_init')
 
 
 def register_ui_init(function):
@@ -103,6 +132,8 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         config.BoolOption("persist", "view_toolbar", True),
         config.BoolOption("persist", "view_file_browser", False),
         config.TextOption("persist", "current_directory", ""),
+        config.FloatOption("persist", "mediaplayer_playback_rate", 1.0),
+        config.IntOption("persist", "mediaplayer_volume", 50),
     ]
 
     def __init__(self, parent=None):
@@ -110,27 +141,33 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.selected_objects = []
         self.ignore_selection_changes = False
         self.toolbar = None
+        self.player = None
+        self.status_indicators = []
+        player = Player(self)
+        if player.available:
+            self.player = player
+            self.player.error.connect(self._on_player_error)
         self.setupUi()
 
     def setupUi(self):
         self.setWindowTitle(_("MusicBrainz Picard"))
         icon = QtGui.QIcon()
-        icon.addFile(":/images/16x16/org.musicbrainz.Picard.png", QtCore.QSize(16, 16))
-        icon.addFile(":/images/24x24/org.musicbrainz.Picard.png", QtCore.QSize(24, 24))
-        icon.addFile(":/images/32x32/org.musicbrainz.Picard.png", QtCore.QSize(32, 32))
-        icon.addFile(":/images/48x48/org.musicbrainz.Picard.png", QtCore.QSize(48, 48))
-        icon.addFile(":/images/128x128/org.musicbrainz.Picard.png", QtCore.QSize(128, 128))
-        icon.addFile(":/images/256x256/org.musicbrainz.Picard.png", QtCore.QSize(256, 256))
+        for size in (16, 24, 32, 48, 128, 256):
+            icon.addFile(
+                ":/images/{size}x{size}/{app_id}.png".format(
+                    size=size, app_id=PICARD_APP_ID),
+                QtCore.QSize(size, size)
+            )
         self.setWindowIcon(icon)
 
-        self.show_close_window = sys.platform == "darwin"
+        self.show_close_window = IS_MACOS
 
         self.create_actions()
         self.create_statusbar()
         self.create_toolbar()
         self.create_menus()
 
-        if sys.platform == "darwin":
+        if IS_MACOS:
             self.setUnifiedTitleAndToolBarOnMac(True)
             self.toolbar.setMovable(False)
             self.search_toolbar.setMovable(False)
@@ -151,8 +188,8 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         if not self.show_cover_art_action.isChecked():
             self.cover_art_box.hide()
 
-        self.logDialog = LogView(self)
-        self.historyDialog = HistoryView(self)
+        self.log_dialog = LogView(self)
+        self.history_dialog = HistoryView(self)
 
         bottomLayout = QtWidgets.QHBoxLayout()
         bottomLayout.setContentsMargins(0, 0, 0, 0)
@@ -175,7 +212,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     def keyPressEvent(self, event):
         # On macOS Command+Backspace triggers the so called "Forward Delete".
         # It should be treated the same as the Delete button.
-        is_forward_delete = sys.platform == 'darwin' and \
+        is_forward_delete = IS_MACOS and \
             event.key() == QtCore.Qt.Key_Backspace and \
             event.modifiers() & QtCore.Qt.ControlModifier
         if event.matches(QtGui.QKeySequence.Delete) or is_forward_delete:
@@ -183,6 +220,8 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
                 self.metadata_box.remove_selected_tags()
             else:
                 self.remove()
+        elif event.matches(QtGui.QKeySequence.Find):
+            self.search_edit.setFocus(True)
         else:
             super().keyPressEvent(event)
 
@@ -193,13 +232,24 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
             self.auto_update_check()
         self.metadata_box.restore_state()
 
+    def showEvent(self, event):
+        super().showEvent(event)
+        if DesktopStatusIndicator:
+            self.register_status_indicator(DesktopStatusIndicator(self.windowHandle()))
+
     def closeEvent(self, event):
         if config.setting["quit_confirmation"] and not self.show_quit_confirmation():
             event.ignore()
             return
+        if self.player:
+            config.persist['mediaplayer_playback_rate'] = self.player.playback_rate()
+            config.persist['mediaplayer_volume'] = self.player.volume()
         self.saveWindowState()
         event.accept()
 
+    def register_status_indicator(self, indicator):
+        self.status_indicators.append(indicator)
+
     def show_quit_confirmation(self):
         unsaved_files = sum(a.get_num_unsaved_files() for a in self.tagger.albums.values())
         QMessageBox = QtWidgets.QMessageBox
@@ -229,6 +279,8 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         config.persist["window_state"] = self.saveState()
         isMaximized = int(self.windowState()) & QtCore.Qt.WindowMaximized != 0
         self.save_geometry()
+        self.log_dialog.save_geometry()
+        self.history_dialog.save_geometry()
         config.persist["window_maximized"] = isMaximized
         config.persist["view_cover_art"] = self.show_cover_art_action.isChecked()
         config.persist["view_toolbar"] = self.show_toolbar_action.isChecked()
@@ -254,7 +306,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     def create_statusbar(self):
         """Creates a new status bar."""
         self.statusBar().showMessage(_("Ready"))
-        self.infostatus = InfoStatus(self)
+        infostatus = InfoStatus(self)
         self.listening_label = QtWidgets.QLabel()
         self.listening_label.setVisible(False)
         self.listening_label.setToolTip("<qt/>" + _(
@@ -262,20 +314,24 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
             "you \"Search\" or \"Open in Browser\" from Picard, clicking the "
             "\"Tagger\" button on the web page loads the release into Picard."
         ))
-        self.statusBar().addPermanentWidget(self.infostatus)
+        self.statusBar().addPermanentWidget(infostatus)
         self.statusBar().addPermanentWidget(self.listening_label)
         self.tagger.tagger_stats_changed.connect(self.update_statusbar_stats)
         self.tagger.listen_port_changed.connect(self.update_statusbar_listen_port)
+        self.register_status_indicator(infostatus)
         self.update_statusbar_stats()
 
     @throttle(100)
     def update_statusbar_stats(self):
         """Updates the status bar information."""
-        self.infostatus.setFiles(len(self.tagger.files))
-        self.infostatus.setAlbums(len(self.tagger.albums))
-        self.infostatus.setPendingFiles(File.num_pending_files)
-        ws = self.tagger.webservice
-        self.infostatus.setPendingRequests(ws.num_pending_web_requests)
+        total_files = len(self.tagger.files)
+        total_albums = len(self.tagger.albums)
+        pending_files = File.num_pending_files
+        pending_requests = self.tagger.webservice.num_pending_web_requests
+
+        for indicator in self.status_indicators:
+            indicator.update(files=total_files, albums=total_albums,
+                pending_files=pending_files, pending_requests=pending_requests)
 
     def update_statusbar_listen_port(self, listen_port):
         if listen_port:
@@ -379,10 +435,10 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.add_files_action.setShortcut(QtGui.QKeySequence.Open)
         self.add_files_action.triggered.connect(self.add_files)
 
-        self.add_directory_action = QtWidgets.QAction(icontheme.lookup('folder'), _("A&dd Folder..."), self)
+        self.add_directory_action = QtWidgets.QAction(icontheme.lookup('folder'), _("Add Fold&er..."), self)
         self.add_directory_action.setStatusTip(_("Add a folder to the tagger"))
         # TR: Keyboard shortcut for "Add Directory..."
-        self.add_directory_action.setShortcut(QtGui.QKeySequence(_("Ctrl+D")))
+        self.add_directory_action.setShortcut(QtGui.QKeySequence(_("Ctrl+E")))
         self.add_directory_action.triggered.connect(self.add_directory)
 
         if self.show_close_window:
@@ -463,18 +519,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         if discid is None:
             log.warning("CDROM: discid library not found - Lookup CD functionality disabled")
         else:
-            drives = get_cdrom_drives()
-            if not drives:
-                log.warning("CDROM: No CD-ROM drives found - Lookup CD functionality disabled")
-            else:
-                shortcut_drive = config.setting["cd_lookup_device"].split(",")[0] if len(drives) > 1 else ""
-                self.cd_lookup_action.setEnabled(True)
-                for drive in drives:
-                    action = self.cd_lookup_menu.addAction(drive)
-                    if drive == shortcut_drive:
-                        # Clear existing shortcode on main action and assign it to sub-action
-                        self.cd_lookup_action.setShortcut(QtGui.QKeySequence())
-                        action.setShortcut(QtGui.QKeySequence(_("Ctrl+K")))
+            thread.run_task(get_cdrom_drives, self._update_cd_lookup_actions)
 
         self.analyze_action = QtWidgets.QAction(icontheme.lookup('picard-analyze'), _("&Scan"), self)
         self.analyze_action.setStatusTip(_("Use AcoustID audio fingerprint to identify the files by the actual music, even if they have no metadata"))
@@ -484,6 +529,14 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.analyze_action.setShortcut(QtGui.QKeySequence(_("Ctrl+Y")))
         self.analyze_action.triggered.connect(self.analyze)
 
+        self.generate_fingerprints_action = QtWidgets.QAction(icontheme.lookup('fingerprint'), _("&Generate AcoustID Fingerprints"), self)
+        self.generate_fingerprints_action.setIconText(_("Generate Fingerprints"))
+        self.generate_fingerprints_action.setStatusTip(_("Generate the AcoustID audio fingerprints for the selected files without doing a lookup"))
+        self.generate_fingerprints_action.setEnabled(False)
+        self.generate_fingerprints_action.setToolTip(_('Generate the AcoustID audio fingerprints for the selected files'))
+        self.generate_fingerprints_action.setShortcut(QtGui.QKeySequence(_("Ctrl+Shift+Y")))
+        self.generate_fingerprints_action.triggered.connect(self.generate_fingerprints)
+
         self.cluster_action = QtWidgets.QAction(icontheme.lookup('picard-cluster'), _("Cl&uster"), self)
         self.cluster_action.setStatusTip(_("Cluster files into album clusters"))
         self.cluster_action.setEnabled(False)
@@ -536,7 +589,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.view_log_action = QtWidgets.QAction(_("View &Error/Debug Log"), self)
         self.view_log_action.triggered.connect(self.show_log)
         # TR: Keyboard shortcut for "View Error/Debug Log"
-        self.view_log_action.setShortcut(QtGui.QKeySequence(_("Ctrl+E")))
+        self.view_log_action.setShortcut(QtGui.QKeySequence(_("Ctrl+G")))
 
         self.view_history_action = QtWidgets.QAction(_("View Activity &History"), self)
         self.view_history_action.triggered.connect(self.show_history)
@@ -558,9 +611,41 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.open_folder_action.triggered.connect(self.open_folder)
 
         if self.tagger.autoupdate_enabled:
-            self.check_update_action = QtWidgets.QAction(_("&Check for Update"), self)
+            self.check_update_action = QtWidgets.QAction(_("&Check for Update…"), self)
+            self.check_update_action.setMenuRole(QtWidgets.QAction.ApplicationSpecificRole)
             self.check_update_action.triggered.connect(self.do_update_check)
 
+    def _update_cd_lookup_actions(self, result=None, error=None):
+        if error:
+            log.error("CDROM: Error on CD-ROM drive detection: %r", error)
+        else:
+            self.update_cd_lookup_drives(result)
+
+    def update_cd_lookup_drives(self, drives):
+        if not drives:
+            log.warning("CDROM: No CD-ROM drives found - Lookup CD functionality disabled")
+        else:
+            shortcut_drive = config.setting["cd_lookup_device"].split(",")[0] if len(drives) > 1 else ""
+            self.cd_lookup_action.setEnabled(discid is not None)
+            self.cd_lookup_menu.clear()
+            for drive in drives:
+                action = self.cd_lookup_menu.addAction(drive)
+                action.setData(drive)
+                if drive == shortcut_drive:
+                    # Clear existing shortcode on main action and assign it to sub-action
+                    self.cd_lookup_action.setShortcut(QtGui.QKeySequence())
+                    action.setShortcut(QtGui.QKeySequence(_("Ctrl+K")))
+        self._update_cd_lookup_button()
+
+    def _update_cd_lookup_button(self):
+        if len(self.cd_lookup_menu.actions()) > 1:
+            button = self.toolbar.widgetForAction(self.cd_lookup_action)
+            if button:
+                button.setPopupMode(QtWidgets.QToolButton.MenuButtonPopup)
+            self.cd_lookup_action.setMenu(self.cd_lookup_menu)
+        else:
+            self.cd_lookup_action.setMenu(None)
+
     def toggle_rename_files(self, checked):
         config.setting["rename_files"] = checked
 
@@ -611,6 +696,8 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         menu.addSeparator()
         menu.addAction(self.show_toolbar_action)
         menu.addAction(self.search_toolbar_toggle_action)
+        if self.player:
+            menu.addAction(self.player_toolbar_toggle_action)
         menu = self.menuBar().addMenu(_("&Options"))
         menu.addAction(self.enable_renaming_action)
         menu.addAction(self.enable_moving_action)
@@ -619,15 +706,13 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         menu.addAction(self.options_action)
         menu = self.menuBar().addMenu(_("&Tools"))
         menu.addAction(self.refresh_action)
-        if len(self.cd_lookup_menu.actions()) > 1:
-            menu.addMenu(self.cd_lookup_menu)
-        else:
-            menu.addAction(self.cd_lookup_action)
+        menu.addAction(self.cd_lookup_action)
         menu.addAction(self.autotag_action)
         menu.addAction(self.analyze_action)
         menu.addAction(self.cluster_action)
         menu.addAction(self.browser_lookup_action)
         menu.addSeparator()
+        menu.addAction(self.generate_fingerprints_action)
         menu.addAction(self.tags_from_filenames_action)
         menu.addAction(self.open_collection_in_browser_action)
         self.menuBar().addSeparator()
@@ -649,11 +734,17 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
     def update_toolbar_style(self):
         if config.setting["toolbar_show_labels"]:
             self.toolbar.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
+            if self.player:
+                self.player.toolbar.setToolButtonStyle(QtCore.Qt.ToolButtonTextUnderIcon)
         else:
             self.toolbar.setToolButtonStyle(QtCore.Qt.ToolButtonIconOnly)
+            if self.player:
+                self.player.toolbar.setToolButtonStyle(QtCore.Qt.ToolButtonIconOnly)
 
     def create_toolbar(self):
         self.create_search_toolbar()
+        if self.player:
+            self.create_player_toolbar()
         self.create_action_toolbar()
 
     def create_action_toolbar(self):
@@ -674,10 +765,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         for action in config.setting['toolbar_layout']:
             if action == 'cd_lookup_action':
                 add_toolbar_action(self.cd_lookup_action)
-                if len(self.cd_lookup_menu.actions()) > 1:
-                    button = toolbar.widgetForAction(self.cd_lookup_action)
-                    button.setPopupMode(QtWidgets.QToolButton.MenuButtonPopup)
-                    button.setMenu(self.cd_lookup_menu)
+                self._update_cd_lookup_button()
             elif action == 'separator':
                 toolbar.addSeparator()
             else:
@@ -687,6 +775,13 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
                     log.warning('Warning: Unknown action name "%r" found in config. Ignored.', action)
         self.show_toolbar()
 
+    def create_player_toolbar(self):
+        """"Create a toolbar with internal player control elements"""
+        toolbar = self.player.create_toolbar()
+        self.addToolBar(QtCore.Qt.BottomToolBarArea, toolbar)
+        self.player_toolbar_toggle_action = toolbar.toggleViewAction()
+        toolbar.hide()  # Hide by default
+
     def create_search_toolbar(self):
         self.search_toolbar = toolbar = self.addToolBar(_("Search"))
         self.search_toolbar_toggle_action = self.search_toolbar.toggleViewAction()
@@ -698,7 +793,8 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.search_combo.addItem(_("Artist"), "artist")
         self.search_combo.addItem(_("Track"), "track")
         hbox.addWidget(self.search_combo, 0)
-        self.search_edit = ButtonLineEdit(search_panel)
+        self.search_edit = QtWidgets.QLineEdit(search_panel)
+        self.search_edit.setClearButtonEnabled(True)
         self.search_edit.returnPressed.connect(self.trigger_search_action)
         self.search_edit.textChanged.connect(self.enable_search)
         hbox.addWidget(self.search_edit, 0)
@@ -844,24 +940,23 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.tagger.activeWindow().close()
 
     def show_about(self):
-        self.show_options("about")
+        return self.show_options("about")
 
     def show_options(self, page=None):
-        dialog = OptionsDialog(page, self)
-        dialog.exec_()
+        return OptionsDialog.show_instance(page, self)
 
     def show_help(self):
         webbrowser2.goto('documentation')
 
     def show_log(self):
-        self.logDialog.show()
-        self.logDialog.raise_()
-        self.logDialog.activateWindow()
+        self.log_dialog.show()
+        self.log_dialog.raise_()
+        self.log_dialog.activateWindow()
 
     def show_history(self):
-        self.historyDialog.show()
-        self.historyDialog.raise_()
-        self.historyDialog.activateWindow()
+        self.history_dialog.show()
+        self.history_dialog.raise_()
+        self.history_dialog.activateWindow()
 
     def open_bug_report(self):
         webbrowser2.goto('troubleshooting')
@@ -881,12 +976,16 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.panel.remove(self.selected_objects)
 
     def analyze(self):
-        if not config.setting['fingerprinting_system']:
-            if self.show_analyze_settings_info():
-                self.show_options("fingerprinting")
-            if not config.setting['fingerprinting_system']:
-                return
-        return self.tagger.analyze(self.selected_objects)
+        def callback(fingerprinting_system):
+            if fingerprinting_system:
+                self.tagger.analyze(self.selected_objects)
+        self._ensure_fingerprinting_configured(callback)
+
+    def generate_fingerprints(self):
+        def callback(fingerprinting_system):
+            if fingerprinting_system:
+                self.tagger.generate_fingerprints(self.selected_objects)
+        self._ensure_fingerprinting_configured(callback)
 
     def _openUrl(self, url):
         return QtCore.QUrl.fromLocalFile(url)
@@ -896,13 +995,26 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         for file in files:
             QtGui.QDesktopServices.openUrl(self._openUrl(file.filename))
 
+    def _on_player_error(self, error, msg):
+        self.set_statusbar_message(msg, echo=log.warning, translate=None)
+
     def open_folder(self):
         files = self.tagger.get_files_from_objects(self.selected_objects)
         folders = set([os.path.dirname(f.filename) for f in files])
         for folder in folders:
             QtGui.QDesktopServices.openUrl(self._openUrl(folder))
 
-    def show_analyze_settings_info(self):
+    def _ensure_fingerprinting_configured(self, callback):
+        def on_finished(result):
+            callback(config.setting['fingerprinting_system'])
+        if not config.setting['fingerprinting_system']:
+            if self._show_analyze_settings_info():
+                dialog = self.show_options("fingerprinting")
+                dialog.finished.connect(on_finished)
+        else:
+            callback(config.setting['fingerprinting_system'])
+
+    def _show_analyze_settings_info(self):
         ret = QtWidgets.QMessageBox.question(self,
             _("Configuration Required"),
             _("Audio fingerprinting is not yet configured. Would you like to configure it now?"),
@@ -982,6 +1094,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.save_action.setEnabled(can_save)
         self.view_info_action.setEnabled(can_view_info)
         self.analyze_action.setEnabled(can_analyze)
+        self.generate_fingerprints_action.setEnabled(have_files)
         self.refresh_action.setEnabled(can_refresh)
         self.autotag_action.setEnabled(can_autotag)
         self.browser_lookup_action.setEnabled(can_browser_lookup)
@@ -1010,6 +1123,9 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         # Clear any existing status bar messages
         self.set_statusbar_message("")
 
+        if self.player:
+            self.player.set_objects(self.selected_objects)
+
         if len(objects) == 1:
             obj = list(objects)[0]
             if isinstance(obj, File):
