@@ -1,7 +1,12 @@
 # -*- coding: utf-8 -*-
 #
 # Picard, the next-generation MusicBrainz tagger
-# Copyright (C) 2006 Lukáš Lalinský
+#
+# Copyright (C) 2006-2007 Lukáš Lalinský
+# Copyright (C) 2009 Nikolai Prokoschenko
+# Copyright (C) 2009, 2019-2020 Philipp Wolfer
+# Copyright (C) 2013, 2015, 2018-2019 Laurent Monin
+# Copyright (C) 2016-2017 Sambhav Kothari
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,8 +22,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
+
 import re
-from PyQt4 import QtGui
+
+from PyQt5 import QtWidgets
+
 from picard import config
 from picard.plugin import ExtensionPoint
 
@@ -30,15 +38,27 @@ class OptionsCheckError(Exception):
         self.info = info
 
 
-class OptionsPage(QtGui.QWidget):
+class OptionsPage(QtWidgets.QWidget):
 
     PARENT = None
     SORT_ORDER = 1000
     ACTIVE = True
     STYLESHEET_ERROR = "QWidget { background-color: #f55; color: white; font-weight:bold }"
+    STYLESHEET = "QLabel { qproperty-wordWrap: true; }"
 
-    def info(self):
-        raise NotImplementedError
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.setStyleSheet(self.STYLESHEET)
+
+        # Keep track whether the options page has been destroyed to avoid
+        # trying to update deleted UI widgets after plugin list refresh.
+        self.deleted = False
+
+        # The on destroyed cannot be created as a method on this class or it will never get called.
+        # See https://stackoverflow.com/questions/16842955/widgets-destroyed-signal-is-not-fired-pyqt
+        def on_destroyed(obj=None):
+            self.deleted = True
+        self.destroyed.connect(on_destroyed)
 
     def check(self):
         pass
@@ -65,7 +85,7 @@ class OptionsPage(QtGui.QWidget):
             config.setting[key] = old_options[key]
 
     def display_error(self, error):
-        dialog = QtGui.QMessageBox(QtGui.QMessageBox.Warning, error.title, error.info, QtGui.QMessageBox.Ok, self)
+        dialog = QtWidgets.QMessageBox(QtWidgets.QMessageBox.Warning, error.title, error.info, QtWidgets.QMessageBox.Ok, self)
         dialog.exec_()
 
     def init_regex_checker(self, regex_edit, regex_error):
@@ -78,7 +98,7 @@ class OptionsPage(QtGui.QWidget):
 
         def check():
             try:
-                re.compile(unicode(regex_edit.text()))
+                re.compile(regex_edit.text())
             except re.error as e:
                 raise OptionsCheckError(_("Regex Error"), str(e))
 
@@ -94,7 +114,7 @@ class OptionsPage(QtGui.QWidget):
         regex_edit.textChanged.connect(live_checker)
 
 
-_pages = ExtensionPoint()
+_pages = ExtensionPoint(label='pages')
 
 
 def register_options_page(page_class):
