@@ -1,7 +1,14 @@
 # -*- coding: utf-8 -*-
 #
 # Picard, the next-generation MusicBrainz tagger
-# Copyright (C) 2006 Lukáš Lalinský
+#
+# Copyright (C) 2006-2007 Lukáš Lalinský
+# Copyright (C) 2010, 2018-2019 Philipp Wolfer
+# Copyright (C) 2012, 2014 Wieland Hoffmann
+# Copyright (C) 2012-2014 Michael Wiencek
+# Copyright (C) 2013-2015, 2018-2019 Laurent Monin
+# Copyright (C) 2016-2018 Sambhav Kothari
+# Copyright (C) 2017 Suhas
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,15 +24,17 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-from PyQt4 import QtCore, QtGui
+
 from picard import config
-from picard.ui.options import OptionsPage, register_options_page
-from picard.ui.ui_options_cover import Ui_CoverOptionsPage
-from picard.coverart.providers import cover_art_providers, is_provider_enabled
-from picard.ui.sortablecheckboxlist import (
-    SortableCheckboxListWidget,
-    SortableCheckboxListItem
+from picard.coverart.providers import cover_art_providers
+
+from picard.ui.checkbox_list_item import CheckboxListItem
+from picard.ui.moveable_list_view import MoveableListView
+from picard.ui.options import (
+    OptionsPage,
+    register_options_page,
 )
+from picard.ui.ui_options_cover import Ui_CoverOptionsPage
 
 
 class CoverOptionsPage(OptionsPage):
@@ -52,35 +61,31 @@ class CoverOptionsPage(OptionsPage):
     ]
 
     def __init__(self, parent=None):
-        super(CoverOptionsPage, self).__init__(parent)
+        super().__init__(parent)
         self.ui = Ui_CoverOptionsPage()
         self.ui.setupUi(self)
         self.ui.save_images_to_files.clicked.connect(self.update_filename)
         self.ui.save_images_to_tags.clicked.connect(self.update_save_images_to_tags)
-        self.provider_list_widget = SortableCheckboxListWidget()
-        self.ui.ca_providers_list.insertWidget(0, self.provider_list_widget)
-        self.ca_providers = []
+        self.move_view = MoveableListView(self.ui.ca_providers_list, self.ui.up_button,
+                                          self.ui.down_button)
 
     def load_cover_art_providers(self):
         """Load available providers, initialize provider-specific options, restore state of each
         """
-        providers = cover_art_providers()
-        for provider in providers:
-            try:
-                title = _(provider.TITLE)
-            except AttributeError:
-                title = provider.NAME
-            checked = is_provider_enabled(provider.NAME)
-            self.provider_list_widget.addItem(SortableCheckboxListItem(title, checked=checked, data=provider.NAME))
-
-        def update_providers_options(items):
-            self.ca_providers = [(item.data, item.checked) for item in items]
-        self.provider_list_widget.changed.connect(update_providers_options)
+        for p in cover_art_providers():
+            self.ui.ca_providers_list.addItem(CheckboxListItem(_(p.title), checked=p.enabled, data=p.name))
 
     def restore_defaults(self):
         # Remove previous entries
-        self.provider_list_widget.clear()
-        super(CoverOptionsPage, self).restore_defaults()
+        self.ui.ca_providers_list.clear()
+        super().restore_defaults()
+
+    def ca_providers(self):
+        items = []
+        for i in range(self.ui.ca_providers_list.count()):
+            item = self.ui.ca_providers_list.item(i)
+            items.append((item.data, item.checked))
+        return items
 
     def load(self):
         self.ui.save_images_to_tags.setChecked(config.setting["save_images_to_tags"])
@@ -88,29 +93,37 @@ class CoverOptionsPage(OptionsPage):
         self.ui.save_images_to_files.setChecked(config.setting["save_images_to_files"])
         self.ui.cover_image_filename.setText(config.setting["cover_image_filename"])
         self.ui.save_images_overwrite.setChecked(config.setting["save_images_overwrite"])
-        self.ca_providers = config.setting["ca_providers"]
         self.load_cover_art_providers()
+        self.ui.ca_providers_list.setCurrentRow(0)
         self.update_all()
 
     def save(self):
         config.setting["save_images_to_tags"] = self.ui.save_images_to_tags.isChecked()
         config.setting["embed_only_one_front_image"] = self.ui.cb_embed_front_only.isChecked()
         config.setting["save_images_to_files"] = self.ui.save_images_to_files.isChecked()
-        config.setting["cover_image_filename"] = unicode(self.ui.cover_image_filename.text())
+        config.setting["cover_image_filename"] = self.ui.cover_image_filename.text()
         config.setting["save_images_overwrite"] = self.ui.save_images_overwrite.isChecked()
-        config.setting["ca_providers"] = self.ca_providers
+        config.setting["ca_providers"] = self.ca_providers()
 
     def update_all(self):
         self.update_filename()
         self.update_save_images_to_tags()
 
+    def update_ca_providers_groupbox_state(self):
+        files_enabled = self.ui.save_images_to_files.isChecked()
+        tags_enabled = self.ui.save_images_to_tags.isChecked()
+        self.ui.ca_providers_groupbox.setEnabled(files_enabled or tags_enabled)
+
     def update_filename(self):
         enabled = self.ui.save_images_to_files.isChecked()
         self.ui.cover_image_filename.setEnabled(enabled)
         self.ui.save_images_overwrite.setEnabled(enabled)
+        self.update_ca_providers_groupbox_state()
 
     def update_save_images_to_tags(self):
         enabled = self.ui.save_images_to_tags.isChecked()
         self.ui.cb_embed_front_only.setEnabled(enabled)
+        self.update_ca_providers_groupbox_state()
+
 
 register_options_page(CoverOptionsPage)
