@@ -1,10 +1,19 @@
 # -*- coding: utf-8 -*-
 #
 # Picard, the next-generation MusicBrainz tagger
+#
 # Copyright (C) 2007 Lukáš Lalinský
 # Copyright (C) 2009 Carlin Mangar
+# Copyright (C) 2009, 2018-2020 Philipp Wolfer
+# Copyright (C) 2011-2013 Michael Wiencek
+# Copyright (C) 2013, 2015, 2018-2019 Laurent Monin
+# Copyright (C) 2013, 2017 Sophist-UK
 # Copyright (C) 2014 Shadab Zafar
-# Copyright (C) 2015 Laurent Monin
+# Copyright (C) 2015, 2017 Wieland Hoffmann
+# Copyright (C) 2016-2018 Sambhav Kothari
+# Copyright (C) 2017 Suhas
+# Copyright (C) 2018 Vishal Choudhary
+# Copyright (C) 2018 yagyanshbhatia
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -20,10 +29,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
+
 from functools import partial
 from operator import attrgetter
 import os.path
-import sys
 
 from PyQt5 import (
     QtCore,
@@ -49,6 +58,7 @@ from picard.ui.options import (
 )
 from picard.ui.ui_options_plugins import Ui_PluginsOptionsPage
 
+
 COLUMN_NAME, COLUMN_VERSION, COLUMN_ACTIONS = range(3)
 
 
@@ -113,6 +123,9 @@ class PluginTreeWidgetItem(HashableTreeWidgetItem):
     @staticmethod
     def set_icon(button, stdicon):
         button.setIcon(button.style().standardIcon(getattr(QtWidgets.QStyle, stdicon)))
+        # Workaround for Qt sometimes not updating the icon.
+        # See https://tickets.metabrainz.org/browse/PICARD-1647
+        button.repaint()
 
     def show_install(self, button, mode):
         if mode == 'hide':
@@ -127,7 +140,7 @@ class PluginTreeWidgetItem(HashableTreeWidgetItem):
             button.hide()
         else:
             button.show()
-            button.setToolTip(_("Download and upgrade plugin to version %s") % self.new_version)
+            button.setToolTip(_("Download and upgrade plugin to version %s") % self.new_version.to_string(short=True))
             self.set_icon(button, 'SP_BrowserReload')
 
     def show_enable(self, button, mode):
@@ -316,7 +329,7 @@ class PluginsOptionsPage(OptionsPage):
             new_version = None
             if plugin.module_name in available_plugins:
                 latest = available_plugins[plugin.module_name]
-                if latest.split('.') > plugin.version.split('.'):
+                if latest > plugin.version:
                     new_version = latest
             self.update_plugin_item(None, plugin,
                                     enabled=self.is_plugin_enabled(plugin),
@@ -369,6 +382,8 @@ class PluginsOptionsPage(OptionsPage):
                     self.set_current_item(item, scroll=True)
 
     def _reload(self):
+        if self.deleted:
+            return
         self._remove_all()
         self._populate()
         self._restore_plugins_states()
@@ -387,7 +402,7 @@ class PluginsOptionsPage(OptionsPage):
         QtWidgets.QMessageBox.critical(
             self,
             _("Plugin '%s'") % plugin_name,
-            _("An error occured while loading the plugin '%s':\n\n%s") % (plugin_name, error)
+            _("An error occurred while loading the plugin '%s':\n\n%s") % (plugin_name, error)
         )
 
     def plugin_installed(self, plugin):
@@ -467,9 +482,10 @@ class PluginsOptionsPage(OptionsPage):
 
         def update_text():
             if item.new_version is not None:
-                version = "%s → %s" % (plugin.version, item.new_version)
+                version = "%s → %s" % (plugin.version.to_string(short=True),
+                                       item.new_version.to_string(short=True))
             else:
-                version = plugin.version
+                version = plugin.version.to_string(short=True)
 
             if item.installed_font is None:
                 item.installed_font = item.font(COLUMN_NAME)
@@ -523,7 +539,8 @@ class PluginsOptionsPage(OptionsPage):
 
         if item.is_installed:
             item.buttons['install'].mode('hide')
-            item.buttons['uninstall'].mode('show')
+            item.buttons['uninstall'].mode(
+                'show' if plugin.is_user_installed else 'hide')
             item.enable(enabled, greyout=False)
 
             def uninstall_processor():
@@ -555,7 +572,7 @@ class PluginsOptionsPage(OptionsPage):
                 return int(elem)
             except ValueError:
                 return 0
-        item.setSortData(COLUMN_VERSION, tuple(v2int(e) for e in plugin.version.split('.')))
+        item.setSortData(COLUMN_VERSION, plugin.version)
 
         return item
 
@@ -633,11 +650,7 @@ class PluginsOptionsPage(OptionsPage):
 
     @staticmethod
     def open_plugin_dir():
-        if sys.platform == 'win32':
-            url = 'file:///' + USER_PLUGIN_DIR
-        else:
-            url = 'file://' + USER_PLUGIN_DIR
-        QtGui.QDesktopServices.openUrl(QtCore.QUrl(url, QtCore.QUrl.TolerantMode))
+        QtGui.QDesktopServices.openUrl(QtCore.QUrl.fromLocalFile(USER_PLUGIN_DIR))
 
     def mimeTypes(self):
         return ["text/uri-list"]
