@@ -1,8 +1,20 @@
 # -*- coding: utf-8 -*-
 #
 # Picard, the next-generation MusicBrainz tagger
-# Copyright (C) 2006-2008 Lukáš Lalinský
-# Copyright (C) 2009 Nikolai Prokoschenko
+#
+# Copyright (C) 2006-2008, 2011 Lukáš Lalinský
+# Copyright (C) 2008-2009 Nikolai Prokoschenko
+# Copyright (C) 2009-2010, 2014-2015, 2018-2019 Philipp Wolfer
+# Copyright (C) 2011-2013 Michael Wiencek
+# Copyright (C) 2011-2013 Wieland Hoffmann
+# Copyright (C) 2013 Calvin Walton
+# Copyright (C) 2013 Ionuț Ciocîrlan
+# Copyright (C) 2013-2014 Sophist-UK
+# Copyright (C) 2013-2015, 2018-2019 Laurent Monin
+# Copyright (C) 2015 Alex Berman
+# Copyright (C) 2015 Ohm Patel
+# Copyright (C) 2016 Suhas
+# Copyright (C) 2016-2017 Sambhav Kothari
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -18,27 +30,45 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-import os.path
-import sys
+
 from functools import partial
-from PyQt4 import QtGui
-from PyQt4.QtGui import QPalette, QColor
+import os.path
+
+from PyQt5 import QtWidgets
+from PyQt5.QtCore import QStandardPaths
+from PyQt5.QtGui import (
+    QFont,
+    QPalette,
+)
+
 from picard import config
-from picard.const import PICARD_URLS
+from picard.const import (
+    DEFAULT_FILE_NAMING_FORMAT,
+    PICARD_URLS,
+)
+from picard.const.sys import IS_WIN
 from picard.file import File
-from picard.script import ScriptParser, SyntaxError, ScriptError, UnknownFunction
-from picard.ui.options import OptionsPage, OptionsCheckError, register_options_page
+from picard.script import (
+    ScriptError,
+    ScriptParser,
+)
+from picard.util.settingsoverride import SettingsOverride
+
+from picard.ui.options import (
+    OptionsCheckError,
+    OptionsPage,
+    register_options_page,
+)
+from picard.ui.options.scripting import (
+    ScriptCheckError,
+    TaggerScriptSyntaxHighlighter,
+)
 from picard.ui.ui_options_renaming import Ui_RenamingOptionsPage
 from picard.ui.util import enabledSlot
-from picard.ui.options.scripting import TaggerScriptSyntaxHighlighter
 
 
-_DEFAULT_FILE_NAMING_FORMAT = "$if2(%albumartist%,%artist%)/" \
-    "$if($ne(%albumartist%,),%album%/,)" \
-    "$if($gt(%totaldiscs%,1),%discnumber%-,)" \
-    "$if($ne(%albumartist%,),$num(%tracknumber%,2) ,)" \
-    "$if(%_multiartist%,%artist% - ,)" \
-    "%title%"
+_default_music_dir = QStandardPaths.writableLocation(QStandardPaths.MusicLocation)
+
 
 class RenamingOptionsPage(OptionsPage):
 
@@ -55,17 +85,17 @@ class RenamingOptionsPage(OptionsPage):
         config.TextOption(
             "setting",
             "file_naming_format",
-            _DEFAULT_FILE_NAMING_FORMAT,
+            DEFAULT_FILE_NAMING_FORMAT,
         ),
         config.BoolOption("setting", "move_files", False),
-        config.TextOption("setting", "move_files_to", ""),
+        config.TextOption("setting", "move_files_to", _default_music_dir),
         config.BoolOption("setting", "move_additional_files", False),
         config.TextOption("setting", "move_additional_files_pattern", "*.jpg *.png"),
         config.BoolOption("setting", "delete_empty_dirs", True),
     ]
 
     def __init__(self, parent=None):
-        super(RenamingOptionsPage, self).__init__(parent)
+        super().__init__(parent)
         self.ui = Ui_RenamingOptionsPage()
         self.ui.setupUi(self)
 
@@ -92,13 +122,17 @@ class RenamingOptionsPage(OptionsPage):
         self.highlighter = TaggerScriptSyntaxHighlighter(self.ui.file_naming_format.document())
         self.ui.move_files_to_browse.clicked.connect(self.move_files_to_browse)
 
-        textEdit = self.ui.file_naming_format
-        self.textEditPaletteNormal = textEdit.palette()
-        self.textEditPaletteReadOnly = QPalette(self.textEditPaletteNormal)
-        disabled_color = self.textEditPaletteNormal.color(QPalette.Inactive, QPalette.Window)
-        self.textEditPaletteReadOnly.setColor(QPalette.Disabled, QPalette.Base, disabled_color)
+        script_edit = self.ui.file_naming_format
+        font = QFont('Monospace')
+        font.setStyleHint(QFont.TypeWriter)
+        script_edit.setFont(font)
+        self.script_palette_normal = script_edit.palette()
+        self.script_palette_readonly = QPalette(self.script_palette_normal)
+        disabled_color = self.script_palette_normal.color(QPalette.Inactive, QPalette.Window)
+        self.script_palette_readonly.setColor(QPalette.Disabled, QPalette.Base, disabled_color)
 
     def toggle_file_moving(self, state):
+        self.toggle_file_naming_format()
         self.ui.delete_empty_dirs.setEnabled(state)
         self.ui.move_files_to.setEnabled(state)
         self.ui.move_files_to_browse.setEnabled(state)
@@ -106,46 +140,43 @@ class RenamingOptionsPage(OptionsPage):
         self.ui.move_additional_files_pattern.setEnabled(state)
 
     def toggle_file_renaming(self, state):
+        self.toggle_file_naming_format()
 
-        self.ui.file_naming_format.setEnabled(state)
-        self.ui.file_naming_format_default.setEnabled(state)
-        self.ui.ascii_filenames.setEnabled(state)
-        self.ui.file_naming_format_group.setEnabled(state)
-        if not sys.platform == "win32":
-            self.ui.windows_compatibility.setEnabled(state)
-
-        if self.ui.file_naming_format.isEnabled():
-            self.ui.file_naming_format.setPalette(self.textEditPaletteNormal)
-        else:
-            self.ui.file_naming_format.setPalette(self.textEditPaletteReadOnly)
+    def toggle_file_naming_format(self):
+        active = self.ui.move_files.isChecked() or self.ui.rename_files.isChecked()
+        self.ui.file_naming_format.setEnabled(active)
+        self.ui.file_naming_format_default.setEnabled(active)
+        palette = self.script_palette_normal if active else self.script_palette_readonly
+        self.ui.file_naming_format.setPalette(palette)
 
+        self.ui.ascii_filenames.setEnabled(active)
+        if not IS_WIN:
+            self.ui.windows_compatibility.setEnabled(active)
 
     def check_formats(self):
         self.test()
         self.update_examples()
 
     def _example_to_filename(self, file):
-        settings = {
-            'windows_compatibility': self.ui.windows_compatibility.isChecked(),
+        settings = SettingsOverride(config.setting, {
             'ascii_filenames': self.ui.ascii_filenames.isChecked(),
-            'rename_files': self.ui.rename_files.isChecked(),
+            'file_naming_format': self.ui.file_naming_format.toPlainText(),
             'move_files': self.ui.move_files.isChecked(),
-            'use_va_format': False,  # TODO remove
-            'file_naming_format': unicode(self.ui.file_naming_format.toPlainText()),
-            'move_files_to': os.path.normpath(unicode(self.ui.move_files_to.text()))
-        }
+            'move_files_to': os.path.normpath(self.ui.move_files_to.text()),
+            'rename_files': self.ui.rename_files.isChecked(),
+            'windows_compatibility': self.ui.windows_compatibility.isChecked(),
+        })
+
         try:
             if config.setting["enable_tagger_scripts"]:
                 for s_pos, s_name, s_enabled, s_text in config.setting["list_of_scripts"]:
                     if s_enabled and s_text:
                         parser = ScriptParser()
                         parser.eval(s_text, file.metadata)
-            filename = file._make_filename(file.filename, file.metadata, settings)
+            filename = file.make_filename(file.filename, file.metadata, settings)
             if not settings["move_files"]:
                 return os.path.basename(filename)
             return filename
-        except SyntaxError:
-            return ""
         except ScriptError:
             return ""
         except TypeError:
@@ -160,7 +191,7 @@ class RenamingOptionsPage(OptionsPage):
         self.ui.example_filename_va.setText(example2)
 
     def load(self):
-        if sys.platform == "win32":
+        if IS_WIN:
             self.ui.windows_compatibility.setChecked(True)
             self.ui.windows_compatibility.setEnabled(False)
         else:
@@ -172,7 +203,7 @@ class RenamingOptionsPage(OptionsPage):
         args = {
             "picard-doc-scripting-url": PICARD_URLS['doc_scripting'],
         }
-        text = _(u'<a href="%(picard-doc-scripting-url)s">Open Scripting'
+        text = _('<a href="%(picard-doc-scripting-url)s">Open Scripting'
                  ' Documentation in your browser</a>') % args
         self.ui.file_naming_format_documentation.setText(text)
         self.ui.move_files_to.setText(config.setting["move_files_to"])
@@ -184,34 +215,36 @@ class RenamingOptionsPage(OptionsPage):
 
     def check(self):
         self.check_format()
-        if self.ui.move_files.isChecked() and not unicode(self.ui.move_files_to.text()).strip():
+        if self.ui.move_files.isChecked() and not self.ui.move_files_to.text().strip():
             raise OptionsCheckError(_("Error"), _("The location to move files to must not be empty."))
 
     def check_format(self):
         parser = ScriptParser()
         try:
-            parser.eval(unicode(self.ui.file_naming_format.toPlainText()))
+            parser.eval(self.ui.file_naming_format.toPlainText())
         except Exception as e:
-            raise OptionsCheckError("", str(e))
+            raise ScriptCheckError("", str(e))
         if self.ui.rename_files.isChecked():
-            if not unicode(self.ui.file_naming_format.toPlainText()).strip():
-                raise OptionsCheckError("", _("The file naming format must not be empty."))
+            if not self.ui.file_naming_format.toPlainText().strip():
+                raise ScriptCheckError("", _("The file naming format must not be empty."))
 
     def save(self):
         config.setting["windows_compatibility"] = self.ui.windows_compatibility.isChecked()
         config.setting["ascii_filenames"] = self.ui.ascii_filenames.isChecked()
         config.setting["rename_files"] = self.ui.rename_files.isChecked()
-        config.setting["file_naming_format"] = unicode(self.ui.file_naming_format.toPlainText())
+        config.setting["file_naming_format"] = self.ui.file_naming_format.toPlainText()
         self.tagger.window.enable_renaming_action.setChecked(config.setting["rename_files"])
         config.setting["move_files"] = self.ui.move_files.isChecked()
-        config.setting["move_files_to"] = os.path.normpath(unicode(self.ui.move_files_to.text()))
+        config.setting["move_files_to"] = os.path.normpath(self.ui.move_files_to.text())
         config.setting["move_additional_files"] = self.ui.move_additional_files.isChecked()
-        config.setting["move_additional_files_pattern"] = unicode(self.ui.move_additional_files_pattern.text())
+        config.setting["move_additional_files_pattern"] = self.ui.move_additional_files_pattern.text()
         config.setting["delete_empty_dirs"] = self.ui.delete_empty_dirs.isChecked()
         self.tagger.window.enable_moving_action.setChecked(config.setting["move_files"])
 
     def display_error(self, error):
-        pass
+        # Ignore scripting errors, those are handled inline
+        if not isinstance(error, ScriptCheckError):
+            super().display_error(error)
 
     def set_file_naming_format_default(self):
         self.ui.file_naming_format.setText(self.options[3].default)
@@ -247,38 +280,38 @@ class RenamingOptionsPage(OptionsPage):
     def example_2(self):
         file = File("track05.mp3")
         file.state = File.NORMAL
-        file.metadata['album'] = u"Coup d'État, Volume 1: Ku De Ta / Prologue"
-        file.metadata['title'] = u"I've Got to Learn the Mambo"
-        file.metadata['artist'] = u"Snowboy feat. James Hunter"
-        file.metadata['artistsort'] = u"Snowboy feat. Hunter, James"
+        file.metadata['album'] = "Coup d'État, Volume 1: Ku De Ta / Prologue"
+        file.metadata['title'] = "I've Got to Learn the Mambo"
+        file.metadata['artist'] = "Snowboy feat. James Hunter"
+        file.metadata['artistsort'] = "Snowboy feat. Hunter, James"
         file.metadata['albumartist'] = config.setting['va_name']
         file.metadata['albumartistsort'] = config.setting['va_name']
         file.metadata['tracknumber'] = '5'
         file.metadata['totaltracks'] = '13'
         file.metadata['discnumber'] = '2'
         file.metadata['totaldiscs'] = '2'
-        file.metadata['discsubtitle'] = u"Beat Up"
-        file.metadata['date'] = u'2005-07-04'
-        file.metadata['releasetype'] = [u'album', u'compilation']
-        file.metadata['~primaryreleasetype'] = u'album'
-        file.metadata['~secondaryreleasetype'] = u'compilation'
-        file.metadata['releasestatus'] = u'official'
-        file.metadata['releasecountry'] = u'AU'
+        file.metadata['discsubtitle'] = "Beat Up"
+        file.metadata['date'] = '2005-07-04'
+        file.metadata['releasetype'] = ['album', 'compilation']
+        file.metadata['~primaryreleasetype'] = 'album'
+        file.metadata['~secondaryreleasetype'] = 'compilation'
+        file.metadata['releasestatus'] = 'official'
+        file.metadata['releasecountry'] = 'AU'
         file.metadata['compilation'] = '1'
         file.metadata['~multiartist'] = '1'
         file.metadata['~extension'] = 'mp3'
-        file.metadata['musicbrainz_albumid'] = u'4b50c71e-0a07-46ac-82e4-cb85dc0c9bdd'
-        file.metadata['musicbrainz_recordingid'] = u'b3c487cb-0e55-477d-8df3-01ec6590f099'
-        file.metadata['musicbrainz_releasetrackid'] = u'f8649a05-da39-39ba-957c-7abf8f9012be'
-        file.metadata['musicbrainz_albumartistid'] = u'89ad4ac3-39f7-470e-963a-56509c546377'
-        file.metadata['musicbrainz_artistid'] = [u'7b593455-d207-482c-8c6f-19ce22c94679',
-                                                 u'9e082466-2390-40d1-891e-4803531f43fd']
+        file.metadata['musicbrainz_albumid'] = '4b50c71e-0a07-46ac-82e4-cb85dc0c9bdd'
+        file.metadata['musicbrainz_recordingid'] = 'b3c487cb-0e55-477d-8df3-01ec6590f099'
+        file.metadata['musicbrainz_releasetrackid'] = 'f8649a05-da39-39ba-957c-7abf8f9012be'
+        file.metadata['musicbrainz_albumartistid'] = '89ad4ac3-39f7-470e-963a-56509c546377'
+        file.metadata['musicbrainz_artistid'] = ['7b593455-d207-482c-8c6f-19ce22c94679',
+                                                 '9e082466-2390-40d1-891e-4803531f43fd']
         return file
 
     def move_files_to_browse(self):
-        path = QtGui.QFileDialog.getExistingDirectory(self, "", self.ui.move_files_to.text())
+        path = QtWidgets.QFileDialog.getExistingDirectory(self, "", self.ui.move_files_to.text())
         if path:
-            path = os.path.normpath(unicode(path))
+            path = os.path.normpath(path)
             self.ui.move_files_to.setText(path)
 
     def test(self):
@@ -286,9 +319,10 @@ class RenamingOptionsPage(OptionsPage):
         self.ui.renaming_error.setText("")
         try:
             self.check_format()
-        except OptionsCheckError as e:
+        except ScriptCheckError as e:
             self.ui.renaming_error.setStyleSheet(self.STYLESHEET_ERROR)
             self.ui.renaming_error.setText(e.info)
             return
 
+
 register_options_page(RenamingOptionsPage)
