@@ -1,9 +1,12 @@
 # -*- coding: utf-8 -*-
 #
 # Picard, the next-generation MusicBrainz tagger
+#
 # Copyright (C) 2007 Lukáš Lalinský
 # Copyright (C) 2009 Carlin Mangar
 # Copyright (C) 2017 Sambhav Kothari
+# Copyright (C) 2018-2019 Philipp Wolfer
+# Copyright (C) 2018-2020 Laurent Monin
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -30,18 +33,15 @@ from collections import (
     namedtuple,
 )
 from functools import partial
-import math
 import os.path
 import platform
 import sys
-import time
 
 from PyQt5 import (
     QtCore,
     QtNetwork,
 )
 from PyQt5.QtCore import (
-    QStandardPaths,
     QUrl,
     QUrlQuery,
 )
@@ -54,14 +54,20 @@ from picard import (
     config,
     log,
 )
+from picard.const import (
+    CACHE_DIR,
+    CACHE_SIZE_IN_BYTES,
+)
 from picard.oauth import OAuthManager
 from picard.util import (
     build_qurl,
+    bytes2human,
     parse_json,
 )
 from picard.util.xml import parse_xml
 from picard.webservice import ratecontrol
 
+
 COUNT_REQUESTS_DELAY_MS = 250
 
 TEMP_ERRORS_RETRIES = 5
@@ -143,14 +149,15 @@ class WSRequest(QNetworkRequest):
         self.queryargs = queryargs
         self.priority = priority
         self.important = important
+        self._high_prio_no_cache = False
 
         self.access_token = None
         self._init_headers()
 
-    def _init_headers(self, high_prio_no_cache=False):
+    def _init_headers(self):
         self.setHeader(QNetworkRequest.UserAgentHeader, USER_AGENT_STRING)
 
-        if self.mblogin or high_prio_no_cache:
+        if self.mblogin or self._high_prio_no_cache:
             self.setPriority(QNetworkRequest.HighPriority)
             self.setAttribute(QNetworkRequest.CacheLoadControlAttribute, QNetworkRequest.AlwaysNetwork)
         elif self.cacheloadcontrol is not None:
@@ -216,7 +223,8 @@ class WSGetRequest(WSRequest):
         super().__init__("GET", *args, **kwargs)
 
     def _init_headers(self):
-        super()._init_headers(high_prio_no_cache=self.refresh)
+        self._high_prio_no_cache = self.refresh
+        super()._init_headers()
         self.setAttribute(QNetworkRequest.HttpPipeliningAllowedAttribute,
                           True)
 
@@ -227,7 +235,8 @@ class WSPutRequest(WSRequest):
         super().__init__("PUT", *args, **kwargs)
 
     def _init_headers(self):
-        super()._init_headers(high_prio_no_cache=True)
+        self._high_prio_no_cache = True
+        super()._init_headers()
 
 
 class WSDeleteRequest(WSRequest):
@@ -236,7 +245,8 @@ class WSDeleteRequest(WSRequest):
         super().__init__("DELETE", *args, **kwargs)
 
     def _init_headers(self):
-        super()._init_headers(high_prio_no_cache=True)
+        self._high_prio_no_cache = True
+        super()._init_headers()
 
 
 class WSPostRequest(WSRequest):
@@ -245,7 +255,8 @@ class WSPostRequest(WSRequest):
         super().__init__("POST", *args, **kwargs)
 
     def _init_headers(self):
-        super()._init_headers(high_prio_no_cache=True)
+        self._high_prio_no_cache = True
+        super()._init_headers()
 
 
 class WebService(QtCore.QObject):
@@ -282,15 +293,17 @@ class WebService(QtCore.QObject):
         self._timer_count_pending_requests.setSingleShot(True)
         self._timer_count_pending_requests.timeout.connect(self._count_pending_requests)
 
-    def set_cache(self, cache_size_in_mb=100):
+    def set_cache(self, cache_size_in_bytes=None):
+        if cache_size_in_bytes is None:
+            cache_size_in_bytes = CACHE_SIZE_IN_BYTES
         cache = QtNetwork.QNetworkDiskCache()
-        location = QStandardPaths.writableLocation(QStandardPaths.CacheLocation)
-        cache.setCacheDirectory(os.path.join(location, 'picard'))
-        cache.setMaximumCacheSize(cache_size_in_mb * 1024 * 1024)
+        cache.setCacheDirectory(os.path.join(CACHE_DIR, 'network'))
+        cache.setMaximumCacheSize(cache_size_in_bytes)
         self.manager.setCache(cache)
-        log.debug("NetworkDiskCache dir: %r size: %s / %s",
-                  cache.cacheDirectory(), cache.cacheSize(),
-                  cache.maximumCacheSize())
+        log.debug("NetworkDiskCache dir: %r current size: %s max size: %s",
+                  cache.cacheDirectory(),
+                  bytes2human.decimal(cache.cacheSize(), l10n=False),
+                  bytes2human.decimal(cache.maximumCacheSize(), l10n=False))
 
     def setup_proxy(self):
         proxy = QtNetwork.QNetworkProxy()
@@ -382,14 +395,13 @@ class WebService(QtCore.QObject):
             log.error("Network request error for %s: %s (QT code %d, HTTP code %d)",
                       url, errstr, error, code)
             if (not request.max_retries_reached()
-                        and (code == 503
-                             or code == 429
-                             # Sometimes QT returns a http status code of 200 even when there
-                             # is a service unavailable error. But it returns a QT error code
-                             # of 403 when this happens
-                             or error == 403
-                             )
-                    ):
+                and (code == 503
+                     or code == 429
+                     # Sometimes QT returns a http status code of 200 even when there
+                     # is a service unavailable error. But it returns a QT error code
+                     # of 403 when this happens
+                     or error == 403
+                     )):
                 slow_down = True
                 retries = request.mark_for_retry()
                 log.debug("Retrying %s (#%d)", url, retries)
