@@ -1,7 +1,27 @@
 # -*- coding: utf-8 -*-
+#
+# Picard, the next-generation MusicBrainz tagger
+#
+# Copyright (C) 2019 Zenara Daley
+# Copyright (C) 2019-2020 Philipp Wolfer
+# Copyright (C) 2020 Laurent Monin
+#
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 2
+# of the License, or (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+
+
 import os.path
-import shutil
-from tempfile import mkstemp
 import unittest
 
 import mutagen
@@ -11,6 +31,8 @@ from test.picardtestcase import PicardTestCase
 from picard import config
 import picard.formats
 from picard.formats import ext_to_format
+from picard.formats.mutagenext.aac import AACAPEv2
+from picard.formats.mutagenext.ac3 import AC3APEv2
 from picard.formats.mutagenext.tak import TAK
 from picard.metadata import Metadata
 
@@ -30,6 +52,8 @@ settings = {
     'write_id3v1': True,
     'write_id3v23': False,
     'itunes_compatible_grouping': False,
+    'aac_save_ape': True,
+    'ac3_save_ape': True,
 }
 
 
@@ -55,9 +79,10 @@ def save_and_load_metadata(filename, metadata):
 
 
 def load_raw(filename):
-    f = mutagen.File(filename)
+    # First try special implementations in Picard
+    f = mutagen.File(filename, [AACAPEv2, AC3APEv2, TAK])
     if f is None:
-        f = mutagen.File(filename, [TAK])
+        f = mutagen.File(filename)
     return f
 
 
@@ -80,8 +105,9 @@ TAGS = {
     'barcode': 'Foo',
     'bpm': '80',
     'catalognumber': 'Foo',
-    'comment:': 'Foo',
+    'comment': 'Foo',
     'comment:foo': 'Foo',
+    'comment:deu:foo': 'Foo',
     'compilation': '1',
     'composer': 'Foo',
     'composersort': 'Foo',
@@ -122,6 +148,7 @@ TAGS = {
     'musicip_puid': '00000000-0000-0000-0000-000000000000',
     'originaldate': '1980-01-20',
     'originalyear': '1980',
+    'originalfilename': 'Foo',
     'performer:guest vocal': 'Foo',
     'podcast': '1',
     'podcasturl': 'Foo',
@@ -183,18 +210,6 @@ class CommonTests:
         def copy_of_original_testfile(self):
             return self.copy_file_tmp(self.testfile_path, self.testfile_ext)
 
-        def copy_file_tmp(self, filename, ext):
-            fd, copy = mkstemp(suffix=ext)
-            self.addCleanup(self.remove_file_tmp, copy)
-            os.close(fd)
-            shutil.copy(filename, copy)
-            return copy
-
-        @staticmethod
-        def remove_file_tmp(tmpfile):
-            if os.path.isfile(tmpfile):
-                os.unlink(tmpfile)
-
     class SimpleFormatsTestCase(BaseFileTestCase):
 
         expected_info = {}
@@ -314,6 +329,16 @@ class CommonTests:
                 self.assertIn('~rating', original_metadata.keys())
                 self.assertNotIn('~rating', new_metadata.keys())
 
+        @skipUnlessTestfile
+        def test_delete_lyrics_tags(self):
+            for key in ('lyrics', 'lyrics:'):
+                metadata = Metadata(self.tags)
+                original_metadata = save_and_load_metadata(self.filename, metadata)
+                self.assertIn('lyrics', original_metadata)
+                del metadata[key]
+                new_metadata = save_and_load_metadata(self.filename, metadata)
+                self.assertNotIn('lyrics', new_metadata)
+
         @skipUnlessTestfile
         def test_delete_non_existant_tags(self):
             metadata = Metadata()
@@ -360,7 +385,7 @@ class CommonTests:
             instrument = "accordéon clavier « boutons »"
             artist = "桑山哲也"
             tag = "performer:" + instrument
-            metadata = Metadata({ tag: artist })
+            metadata = Metadata({tag: artist})
             loaded_metadata = save_and_load_metadata(self.filename, metadata)
             self.assertIn(tag, loaded_metadata)
             self.assertEqual(artist, loaded_metadata[tag])
