@@ -1,10 +1,35 @@
-import json
-import os.path
-import shutil
-import sys
-import tempfile
+# -*- coding: utf-8 -*-
+#
+# Picard, the next-generation MusicBrainz tagger
+#
+# Copyright (C) 2013, 2018, 2020 Laurent Monin
+# Copyright (C) 2014 Michael Wiencek
+# Copyright (C) 2017 Sambhav Kothari
+# Copyright (C) 2017 Sophist-UK
+# Copyright (C) 2018 Wieland Hoffmann
+# Copyright (C) 2019 Philipp Wolfer
+#
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 2
+# of the License, or (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
-from test.picardtestcase import PicardTestCase
+
+import os.path
+
+from test.picardtestcase import (
+    PicardTestCase,
+    load_test_json,
+)
 
 from picard import config
 from picard.i18n import setup_gettext
@@ -21,27 +46,16 @@ settings = {
 
 class ReleaseTest(PicardTestCase):
 
-    @staticmethod
-    def load_data(filename):
-        with open(os.path.join('test', 'data', 'ws_data', filename), encoding='utf-8') as f:
-            return json.load(f)
-
     def setUp(self):
         super().setUp()
         # we are using temporary locales for tests
-        self.tmp_path = tempfile.mkdtemp()
-        if sys.hexversion >= 0x020700F0:
-            self.addCleanup(shutil.rmtree, self.tmp_path)
+        self.tmp_path = self.mktmpdir()
         self.localedir = os.path.join(self.tmp_path, 'locale')
         setup_gettext(self.localedir, 'C')
 
-    def tearDown(self):
-        if sys.hexversion < 0x020700F0:
-            shutil.rmtree(self.tmp_path)
-
     def test_1(self):
         config.setting = settings.copy()
-        rlist = self.load_data('release_group_2.json')
+        rlist = load_test_json('release_group_2.json')
         r = ReleaseGroup(1)
         r._parse_versions(rlist)
         self.assertEqual(r.versions[0]['name'],
@@ -53,7 +67,7 @@ class ReleaseTest(PicardTestCase):
 
     def test_2(self):
         config.setting = settings.copy()
-        rlist = self.load_data('release_group_3.json')
+        rlist = load_test_json('release_group_3.json')
         r = ReleaseGroup(1)
         r._parse_versions(rlist)
         self.assertEqual(r.versions[0]['name'],
@@ -63,7 +77,7 @@ class ReleaseTest(PicardTestCase):
 
     def test_3(self):
         config.setting = settings.copy()
-        rlist = self.load_data('release_group_4.json')
+        rlist = load_test_json('release_group_4.json')
         r = ReleaseGroup(1)
         r._parse_versions(rlist)
         self.assertEqual(r.versions[0]['name'],
