@@ -1,4 +1,27 @@
 # -*- coding: utf-8 -*-
+#
+# Picard, the next-generation MusicBrainz tagger
+#
+# Copyright (C) 2013, 2018 Laurent Monin
+# Copyright (C) 2017 Sophist-UK
+# Copyright (C) 2018 Wieland Hoffmann
+# Copyright (C) 2019 Philipp Wolfer
+#
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 2
+# of the License, or (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+
+
 from test.picardtestcase import PicardTestCase
 
 from picard.util import tracknum_from_filename
@@ -68,10 +91,10 @@ class TracknumTest(PicardTestCase):
         self.assertEqual(parse('01_foo.mp3'), 1)
 
     def test_matched_tracknum_20(self):
-        self.assertEqual(parse(u'01ābc.mp3'), 1)
+        self.assertEqual(parse('01ābc.mp3'), 1)
 
     def test_matched_tracknum_21(self):
-        self.assertEqual(parse(u'01abc.mp3'), 1)
+        self.assertEqual(parse('01abc.mp3'), 1)
 
     def test_matched_tracknum_22(self):
         t = u"11 Linda Jones - Things I've Been Through 08.flac"
